%FUNCTION: This function generates a jigsaw shaped puzzle edge given four
    %points from the Voronoi 
    
%INPUT: Vertives on the Voronoi and should be given as (x,y) points
    %start-edge point and initial point of the edge 
    %leftcent- mid point of voronoi and left of start 
    %rightcent- mid point of voronoi and right of start 
    %fin- edge point and final point of the edge 

%OUTPUT: Discrete Bezier curve that forms our puzzle edge

function edge = JigsawEdge(start, leftcent, rightcent, fin)

    %PARAMETRIZING DIAGONAL TO EXTRACT POTENTIAL CONTROL POINTS
    dx=0.01; t=0:dx:1; r=rand;
        
    %START TO FIN
    SFx=(1-t)*(start(1))+t*(fin(1));
    SFy=(1-t)*(start(2))+t*(fin(2));
    SFz=(1-t)*(start(3))+t*(fin(3));
    
    %STRAIGHT EDGE 1
    edge1y=(1-t)*(start(2))+t*(SFy(33));
    edge1x=(1-t)*(start(1))+t*(SFx(33));
    ed1gez=(1-t)*(start(3))+t*(SFz(33));
    
    %CURVE EDGE 2
    %RANDOMLY SELECTING CONTROL POINT AS LEFT OR RIGHT
    if r>0.5
        controlx = leftcent(1); 
        controly = leftcent(2); 
        controlz = leftcent(3);
    else
        controlx = rightcent(1); 
        controly = rightcent(2); 
        controlz = rightcent(3);
    end
    
    edge2x = zeros(33,1);
    edge2y = zeros(33,1); 
    edge2z = zeros(33,1);
    
    for i=1:101
        edge2x(i) = SFx(33)*(1-t(i))^2+2*t(i)*(1-t(i))*controlx+SFx(66)*t(i)^2;
        edge2y(i) = SFy(33)*(1-t(i))^2+2*t(i)*(1-t(i))*controly+SFy(66)*t(i)^2;
        edge2z(i) = SFz(33)*(1-t(i))^2+2*t(i)*(1-t(i))*controlz+SFz(66)*t(i)^2;
    end
    
    %STRAIGHT EDGE 3
    edge3y = (1-t)*(SFy(66))+(t*fin(2));
    edge3x = (1-t)*(SFx(66))+(t*fin(1));
    edge3z = (1-t)*(SFz(66))+(t*fin(3));
    
    %COMPILING EDGES
    edge(:,1) = edge1x; 
    edge(:,2) = edge1y; 
    edge(:,3) = ed1gez;
    
    edge(102:202,1) = edge2x; 
    edge(102:202,2) = edge2y; 
    edge(102:202,3) = edge2z; 
    
    edge(203:303,1) = edge3x; 
    edge(203:303,2) = edge3y; 
    edge(203:303,3) = edge3z;
    
    for p=1:length(edge(:,1))
        [theta(p,1), phi(p,1), rho(p,1)]=cart2sph(edge(p,1),edge(p,2),edge(p,3));
        if rho(p,1)~=1
            rho(p,1)=1;
        end
        [edge(p,3),edge(p,2),edge(p,3)]=sph2cart(theta(p,1),phi(p,1),rho(p,1));
    end
   
end